@@ -1,47 +1,45 @@
 #include <limits.h>
-#include <stdio.h>
 
 #include "cdi.h"  		/* CALENDAR_ */
+#include "calendar.h"
 #include "error.h"
 #include "timebase.h"
 
 
-static int month_360[12] = {30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30};
-static int month_365[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
-static int month_366[12] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+static const int month_360[12] = {30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30};
+static const int month_365[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+static const int month_366[12] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
 
 int calendar_dpy(int calendar)
 {
-  int dpy = 0;
+  int daysperyear = 0;
 
-  if      ( calendar == CALENDAR_360DAYS ) dpy = 360;
-  else if ( calendar == CALENDAR_365DAYS ) dpy = 365;
-  else if ( calendar == CALENDAR_366DAYS ) dpy = 366;
+  if      ( calendar == CALENDAR_360DAYS ) daysperyear = 360;
+  else if ( calendar == CALENDAR_365DAYS ) daysperyear = 365;
+  else if ( calendar == CALENDAR_366DAYS ) daysperyear = 366;
 
-  return (dpy);
+  return daysperyear;
 }
 
 
 int days_per_month(int calendar, int year, int month)
 {
+  int daysperyear = calendar_dpy(calendar);
+
+  const int *dpm;
+  if      ( daysperyear == 360 ) dpm = month_360;
+  else if ( daysperyear == 365 ) dpm = month_365;
+  else                           dpm = month_366;
+
   int dayspermonth = 0;
-  int *dpm = NULL;
-  int dpy;
-
-  dpy = calendar_dpy(calendar);
-
-  if      ( dpy == 360 ) dpm = month_360;
-  else if ( dpy == 365 ) dpm = month_365;
-  else                   dpm = month_366;
-
   if ( month >= 1 && month <= 12 )
     dayspermonth = dpm[month-1];
   /*
   else
     fprintf(stderr, "days_per_month: month %d out of range\n", month);
   */
-  if ( dpy == 0 && month == 2 )
+  if ( daysperyear == 0 && month == 2 )
     {
       if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
 	dayspermonth = 29;
@@ -49,51 +47,36 @@ int days_per_month(int calendar, int year, int month)
 	dayspermonth = 28;
     }
 
-  return (dayspermonth);
+  return dayspermonth;
 }
 
 
 int days_per_year(int calendar, int year)
 {
-  int daysperyear;
-  int dpy;
+  int daysperyear = calendar_dpy(calendar);
 
-  dpy = calendar_dpy(calendar);
-
-  if ( dpy == 0 )
+  if ( daysperyear == 0 )
     {
-      if ( calendar == CALENDAR_STANDARD )
-	{
-	  if ( year == 1582 )
-	    dpy = 355;
-	  else if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
-	    dpy = 366;
-	  else
-	    dpy = 365;
-	}
+      if ( year == 1582 && (calendar == CALENDAR_STANDARD || calendar == CALENDAR_GREGORIAN) )
+        daysperyear = 355;
+      else if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
+        daysperyear = 366;
       else
-	{
-	  if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
-	    dpy = 366;
-	  else
-	    dpy = 365;
-	}
+        daysperyear = 365;
     }
 
-  daysperyear = dpy;
-  
-  return (daysperyear);
+  return daysperyear;
 }
 
 
 static void decode_day(int dpy, int days, int *year, int *month, int *day)
 {
   int i = 0;
-  int *dpm = NULL;
 
   *year = (days-1) / dpy;
   days -= (*year*dpy);
 
+  const int *dpm = NULL;
   if      ( dpy == 360 ) dpm = month_360;
   else if ( dpy == 365 ) dpm = month_365;
   else if ( dpy == 366 ) dpm = month_366;
@@ -110,68 +93,26 @@ static void decode_day(int dpy, int days, int *year, int *month, int *day)
 }
 
 
-static int encode_day(int dpy, int year, int month, int day)
+static int64_t encode_day(int dpy, int year, int month, int day)
 {
-  int i;
-  int *dpm = NULL;
-  long rval = (long)dpy * year + day;
+  int64_t rval = (int64_t)dpy * year + day;
 
+  const int *dpm = NULL;
   if      ( dpy == 360 ) dpm = month_360;
   else if ( dpy == 365 ) dpm = month_365;
   else if ( dpy == 366 ) dpm = month_366;
 
-  if ( dpm ) for ( i = 0; i < month-1; i++ ) rval += dpm[i];
-  if (rval > INT_MAX || rval < INT_MIN)
-    Error("Unhandled date: %ld", rval);
+  if ( dpm ) for ( int i = 0; i < month-1; i++ ) rval += dpm[i];
+  if ( rval > LONG_MAX || rval < LONG_MIN ) Error("Unhandled date: %lld", rval);
 
-  return (int)rval;
-}
-
-
-int date_to_calday(int calendar, int date)
-{
-  int calday;
-  int dpy;
-  int year, month, day;
-
-  dpy = calendar_dpy(calendar);
-
-  cdiDecodeDate(date, &year, &month, &day);
-
-  if ( dpy == 360 || dpy == 365 || dpy == 366 )
-    calday = encode_day(dpy, year, month, day);
-  else
-    calday = encode_julday(calendar, year, month, day);
-
-  return (calday);
-}
-
-
-int calday_to_date(int calendar, int calday)
-{
-  int date;
-  int dpy;
-  int year, month, day;
-
-  dpy = calendar_dpy(calendar);
-
-  if ( dpy == 360 || dpy == 365 || dpy == 366 )
-    decode_day(dpy, calday, &year, &month, &day);
-  else
-    decode_julday(calendar, calday, &year, &month, &day);
-
-  date = cdiEncodeDate(year, month, day);
-
-  return (date);
+  return rval;
 }
 
 
 void encode_caldaysec(int calendar, int year, int month, int day, int hour, int minute, int second,
-		      int *julday, int *secofday)
+		      int64_t *julday, int *secofday)
 {
-  int dpy;
-
-  dpy = calendar_dpy(calendar);
+  int dpy = calendar_dpy(calendar);
 
   if ( dpy == 360 || dpy == 365 || dpy == 366 )
     *julday = encode_day(dpy, year, month, day);
@@ -182,12 +123,10 @@ void encode_caldaysec(int calendar, int year, int month, int day, int hour, int
 }
 
 
-void decode_caldaysec(int calendar, int julday, int secofday, 
+void decode_caldaysec(int calendar, int64_t julday, int secofday,
 		      int *year, int *month, int *day, int *hour, int *minute, int *second)
 {
-  int dpy;
-
-  dpy = calendar_dpy(calendar);
+  int dpy = calendar_dpy(calendar);
 
   if ( dpy == 360 || dpy == 365 || dpy == 366 )
     decode_day(dpy, julday, year, month, day);
@@ -201,12 +140,45 @@ void decode_caldaysec(int calendar, int julday, int secofday,
 
 
 #ifdef TEST
+static int date_to_calday(int calendar, int date)
+{
+  int dpy = calendar_dpy(calendar);
+
+  int year, month, day;
+  cdiDecodeDate(date, &year, &month, &day);
+
+  int calday;
+  if ( dpy == 360 || dpy == 365 || dpy == 366 )
+    calday = encode_day(dpy, year, month, day);
+  else
+    calday = encode_julday(calendar, year, month, day);
+
+  return calday;
+}
+
+
+static int calday_to_date(int calendar, int calday)
+{
+  int year, month, day;
+  int dpy = calendar_dpy(calendar);
+
+  if ( dpy == 360 || dpy == 365 || dpy == 366 )
+    decode_day(dpy, calday, &year, &month, &day);
+  else
+    decode_julday(calendar, calday, &year, &month, &day);
+
+  int date = cdiEncodeDate(year, month, day);
+
+  return date;
+}
+
+
 int main(void)
 {
   int calendar = CALENDAR_STANDARD;
   int nmin;
-  int vdate0, vtime0;
-  int vdate, vtime;
+  int64_t vdate0, vdate;
+  int vtime0, vtime;
   int ijulinc;
   int i, j = 0;
   int year, mon, day, hour, minute, second;
