@@ -1,9 +1,8 @@
-#ifndef _CDF_INT_H
-#define _CDF_INT_H
+#ifndef  CDF_INT_H
+#define  CDF_INT_H
 
-#if  defined  (HAVE_LIBNETCDF)
+#ifdef HAVE_LIBNETCDF
 
-#include <stdlib.h>
 #include <netcdf.h>
 
 void cdf_create (const char *path, int cmode, int *idp);
@@ -85,6 +84,8 @@ void cdf_inq_attlen (int ncid, int varid, const char *name, size_t *lenp);
 void cdf_inq_attname(int ncid, int varid, int attnum, char *name);
 void cdf_inq_attid  (int ncid, int varid, const char *name, int *attnump);
 
+void cdf_def_var_chunking(int ncid, int varid, int storage, const size_t *chunksizesp);
+
 typedef int (*cdi_nc__create_funcp)(const char *path, int cmode,
                                     size_t initialsz, size_t *chunksizehintp,
                                     int *ncidp);
@@ -95,7 +96,7 @@ typedef void (*cdi_cdf_def_var_funcp)(int ncid, const char *name,
 
 #endif
 
-#endif  /* _CDF_INT_H */
+#endif  /* CDF_INT_H */
 /*
  * Local Variables:
  * c-file-style: "Java"
