@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,24 +25,24 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "param_conversion.h"
 #include "pstream_int.h"
 #include "interpol.h"
 #include "grid.h"
-#include <vector>
 
 int
 genThinoutGrid(int gridID1, size_t xinc, size_t yinc)
 {
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
 
   size_t nlon2 = nlon1 / xinc;
   size_t nlat2 = nlat1 / yinc;
   if (nlon1 % xinc) nlon2++;
   if (nlat1 % yinc) nlat2++;
-  size_t gridsize2 = nlon2 * nlat2;
+  const size_t gridsize2 = nlon2 * nlat2;
 
-  int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
+  const int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
@@ -78,20 +78,20 @@ genBoxavgGrid(int gridID1, size_t xinc, size_t yinc)
 {
   size_t i, j, i1;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
 
   size_t nlon2 = nlon1 / xinc;
   size_t nlat2 = nlat1 / yinc;
   if (nlon1 % xinc) nlon2++;
   if (nlat1 % yinc) nlat2++;
-  size_t gridsize2 = nlon2 * nlat2;
+  const size_t gridsize2 = nlon2 * nlat2;
 
-  int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
+  const int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
-  int gridtype = gridInqType(gridID1);
+  const int gridtype = gridInqType(gridID1);
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT)
     {
       std::vector<double> xvals1(nlon1);
@@ -103,7 +103,7 @@ genBoxavgGrid(int gridID1, size_t xinc, size_t yinc)
 
       std::vector<double> grid1_corner_lon, grid1_corner_lat;
       std::vector<double> grid2_corner_lon, grid2_corner_lat;
-      if (gridInqYbounds(gridID1, NULL) && gridInqXbounds(gridID1, NULL))
+      if (gridInqYbounds(gridID1, nullptr) && gridInqXbounds(gridID1, nullptr))
         {
           grid1_corner_lon.resize(2 * nlon1);
           grid1_corner_lat.resize(2 * nlat1);
@@ -161,17 +161,17 @@ genBoxavgGrid(int gridID1, size_t xinc, size_t yinc)
 static void
 boxavg(Field *field1, Field *field2, size_t xinc, size_t yinc)
 {
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
+  const int gridID1 = field1->grid;
+  const int gridID2 = field2->grid;
   double *array1 = field1->ptr;
   double *array2 = field2->ptr;
-  double missval = field1->missval;
+  const double missval = field1->missval;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
 
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
+  const size_t nlon2 = gridInqXsize(gridID2);
+  const size_t nlat2 = gridInqYsize(gridID2);
 
   std::vector<double *> xfield1(nlat1);
   for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1 + ilat * nlon1;
@@ -187,11 +187,11 @@ boxavg(Field *field1, Field *field2, size_t xinc, size_t yinc)
         size_t in = 0;
         for (size_t j = 0; j < yinc; ++j)
           {
-            size_t jj = ilat * yinc + j;
+            const size_t jj = ilat * yinc + j;
             if (jj >= nlat1) break;
             for (size_t i = 0; i < xinc; ++i)
               {
-                size_t ii = ilon * xinc + i;
+                const size_t ii = ilon * xinc + i;
                 if (ii >= nlon1) break;
                 in++;
                 xfield2[ilat][ilon] += xfield1[jj][ii];
@@ -206,17 +206,17 @@ boxavg(Field *field1, Field *field2, size_t xinc, size_t yinc)
 static void
 thinout(Field *field1, Field *field2, int xinc, int yinc)
 {
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
+  const int gridID1 = field1->grid;
+  const int gridID2 = field2->grid;
   double *array1 = field1->ptr;
   double *array2 = field2->ptr;
-  double missval = field1->missval;
+  const double missval = field1->missval;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
 
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
+  const size_t nlon2 = gridInqXsize(gridID2);
+  const size_t nlat2 = gridInqYsize(gridID2);
 
   std::vector<double *> xfield1(nlat1);
   for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1 + ilat * nlon1;
@@ -252,15 +252,15 @@ Intgrid(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int INTGRIDBIL  = cdoOperatorAdd("intgridbil",  0, 0, NULL);
-  int INTGRIDDIS  = cdoOperatorAdd("intgriddis",  0, 0, NULL);
-  int INTGRIDNN   = cdoOperatorAdd("intgridnn",   0, 0, NULL);
-  int INTERPOLATE = cdoOperatorAdd("interpolate", 0, 0, NULL);
-  int BOXAVG      = cdoOperatorAdd("boxavg",      0, 0, NULL);
-  int THINOUT     = cdoOperatorAdd("thinout",     0, 0, NULL);
+  const int INTGRIDBIL  = cdoOperatorAdd("intgridbil",  0, 0, nullptr);
+  const int INTGRIDDIS  = cdoOperatorAdd("intgriddis",  0, 0, nullptr);
+  const int INTGRIDNN   = cdoOperatorAdd("intgridnn",   0, 0, nullptr);
+  const int INTERPOLATE = cdoOperatorAdd("interpolate", 0, 0, nullptr);
+  const int BOXAVG      = cdoOperatorAdd("boxavg",      0, 0, nullptr);
+  const int THINOUT     = cdoOperatorAdd("thinout",     0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   if (operatorID == INTGRIDBIL || operatorID == INTERPOLATE || operatorID == INTGRIDDIS || operatorID == INTGRIDNN)
     {
@@ -275,20 +275,20 @@ Intgrid(void *process)
       yinc = parameter2int(operatorArgv()[1]);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const int ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
       gridID1 = vlistGrid(vlistID1, index);
-      int gridtype = gridInqType(gridID1);
+      const int gridtype = gridInqType(gridID1);
 
       if (operatorID == BOXAVG || operatorID == THINOUT)
         {
@@ -305,24 +305,24 @@ Intgrid(void *process)
         }
       else if (operatorID == INTGRIDBIL || operatorID == INTERPOLATE)
         {
-          bool ldistgen = (grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2));
+          const bool ldistgen = (grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2));
           if (!ldistgen && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN)
             cdoAbort("Interpolation of %s data unsupported!", gridNamePtr(gridtype));
         }
       else if (operatorID == INTGRIDNN || operatorID == INTGRIDDIS)
         {
-          int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID1) : -1;
-          bool lproj4param = (gridtype == GRID_PROJECTION) && grid_has_proj4param(gridID1);
+          const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID1) : -1;
+          const bool lproj4param = (gridtype == GRID_PROJECTION) && grid_has_proj4param(gridID1);
           if (gridtype != GRID_LONLAT && !lproj4param && projtype != CDI_PROJ_RLL && projtype != CDI_PROJ_LAEA
-              && projtype != CDI_PROJ_SINU && projtype != CDI_PROJ_LCC && gridtype != GRID_GAUSSIAN && gridtype != GRID_GME
-              && gridtype != GRID_CURVILINEAR && gridtype != GRID_UNSTRUCTURED)
+              && projtype != CDI_PROJ_SINU && projtype != CDI_PROJ_LCC && projtype != CDI_PROJ_STERE && gridtype != GRID_GAUSSIAN
+              && gridtype != GRID_GME && gridtype != GRID_CURVILINEAR && gridtype != GRID_UNSTRUCTURED)
             cdoAbort("Interpolation of %s data unsupported!", gridNamePtr(gridtype));
         }
 
       vlistChangeGridIndex(vlistID2, index, gridID2);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
   pstreamDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
@@ -332,8 +332,8 @@ Intgrid(void *process)
   std::vector<double> array2(gridsize);
 
   Field field1, field2;
-  field_init(&field1);
-  field_init(&field2);
+  fieldInit(field1);
+  fieldInit(field2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -373,10 +373,10 @@ Intgrid(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
