@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,12 +24,15 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "param_conversion.h"
 #include "cdo_wtime.h"
 #include "pstream_int.h"
 #include "grid.h"
 #include "constants.h"  // planet radius
 #include "pmlist.h"
-#include "cdoOptions.h"
+#include "cdo_options.h"
+#include "progress.h"
+#include "cimdOmp.h"
 
 #include "grid_point_search.h"
 
@@ -61,17 +64,32 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
   uint8_t *mask = vmask.data();
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(array1[i], missval);
 
-  std::vector<double> xvals(gridsize);
-  std::vector<double> yvals(gridsize);
+  int gridtype = gridInqType(gridID);
 
-  if (gridInqType(gridID) == GRID_GME) gridID = gridToUnstructured(gridID, 0);
+  if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
 
-  if (gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
+  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
+  if (gridtype == GRID_UNSTRUCTURED)
+    {
+      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+        {
+          if (gridInqNumber(gridID) > 0)
+            {
+              gridID = referenceToGrid(gridID);
+              if (gridID == -1) cdoAbort("Reference to source grid not found!");
+            }
+        }
+    }
+
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+    cdoAbort("Cell center coordinates missing!");
+
+  std::vector<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, &xvals[0]);
   gridInqYvals(gridID, &yvals[0]);
 
-  /* Convert lat/lon units if required */
+  // Convert lat/lon units if required
   char units[CDI_MAX_NAME];
   gridInqXunits(gridID, units);
   grid_to_radian(units, gridsize, &xvals[0], "grid center lon");
@@ -81,7 +99,7 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  double start = cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   bool xIsCyclic = false;
   size_t dims[2] = { gridsize, 0 };
@@ -89,11 +107,11 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 
   gps->searchRadius = spoint.radius;
 
-  if (cdoVerbose) cdoPrint("Point search created: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Point search created: %.2f seconds", cdo_get_wtime() - start);
 
-  if (cdoVerbose) progressInit();
+  if (Options::cdoVerbose) Progress::init();
 
-  start = cdoVerbose ? cdo_get_wtime() : 0;
+  start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   size_t naddsMax = 0;
   size_t nmissx = 0;
@@ -101,7 +119,7 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 
 #ifdef HAVE_OPENMP4
 #pragma omp parallel for schedule(dynamic) default(none) reduction(+ : nmissx) reduction(max : naddsMax) shared( \
-    findex, cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gps, gridsize, missval)
+    findex, Options::cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gps, gridsize, missval)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -111,7 +129,7 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 #pragma omp atomic
 #endif
       findex++;
-      if (cdoVerbose && cdo_omp_get_thread_num() == 0) progressStatus(0, 1, findex / gridsize);
+      if (Options::cdoVerbose && cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
 
       gridSearchPoint(gps, xvals[i], yvals[i], knnWeights[ompthID]);
 
@@ -131,10 +149,10 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 
   *nmiss = nmissx;
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
-  if (cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
-  if (cdoVerbose) cdoPrint("Max points found: %zu\n", naddsMax);
+  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Max points found: %zu", naddsMax);
 
   if (gps) gridPointSearchDelete(gps);
 
@@ -237,7 +255,7 @@ smooth9(int gridID, double missval, const double *restrict array1, double *restr
               smooth9_sum(((i + 1) * nlon) + j + 1, mask, 0.3, array1, &avg, &divavg);
             }
 
-          if (fabs(divavg) > 0)
+          if (std::fabs(divavg) > 0)
             {
               array2[i * nlon + j] = avg / divavg;
             }
@@ -253,7 +271,7 @@ smooth9(int gridID, double missval, const double *restrict array1, double *restr
 double
 radius_str_to_deg(const char *string)
 {
-  char *endptr = NULL;
+  char *endptr = nullptr;
   double radius = strtod(string, &endptr);
 
   if (*endptr != 0)
@@ -291,35 +309,33 @@ convert_form(const char *formstr)
 static void
 smoothGetParameter(int *xnsmooth, smoothpoint_t *spoint)
 {
-  int pargc = operatorArgc();
+  const int pargc = operatorArgc();
   if (pargc)
     {
-      char **pargv = operatorArgv();
+      const char **pargv = (const char **) operatorArgv();
 
-      list_t *kvlist = list_new(sizeof(keyValues_t *), free_keyval, "SMOOTH");
-      if (kvlist_parse_cmdline(kvlist, pargc, pargv) != 0) cdoAbort("Parse error!");
-      if (cdoVerbose) kvlist_print(kvlist);
+      KVList kvlist;
+      kvlist.name = "SMOOTH";
+      if (kvlist.parseArguments(pargc, pargv) != 0) cdoAbort("Parse error!");
+      if (Options::cdoVerbose) kvlist.print();
 
-      for (listNode_t *kvnode = kvlist->head; kvnode; kvnode = kvnode->next)
+      for (const auto &kv : kvlist)
         {
-          keyValues_t *kv = *(keyValues_t **) kvnode->data;
-          const char *key = kv->key;
-          if (kv->nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv->nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv->values[0];
+          const char *key = kv.key.c_str();
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
+          const char *value = kv.values[0].c_str();
 
           // clang-format off
-          if      (STR_IS_EQ(key, "nsmooth")) *xnsmooth = parameter2int(value);
-          else if (STR_IS_EQ(key, "maxpoints")) spoint->maxpoints = parameter2sizet(value);
-          else if (STR_IS_EQ(key, "weight0")) spoint->weight0 = parameter2double(value);
-          else if (STR_IS_EQ(key, "weightR")) spoint->weightR = parameter2double(value);
-          else if (STR_IS_EQ(key, "radius")) spoint->radius = radius_str_to_deg(value);
-          else if (STR_IS_EQ(key, "form")) spoint->form = convert_form(value);
+          if      (cstrIsEqual(key, "nsmooth")) *xnsmooth = parameter2int(value);
+          else if (cstrIsEqual(key, "maxpoints")) spoint->maxpoints = parameter2sizet(value);
+          else if (cstrIsEqual(key, "weight0")) spoint->weight0 = parameter2double(value);
+          else if (cstrIsEqual(key, "weightR")) spoint->weightR = parameter2double(value);
+          else if (cstrIsEqual(key, "radius")) spoint->radius = radius_str_to_deg(value);
+          else if (cstrIsEqual(key, "form")) spoint->form = convert_form(value);
           else cdoAbort("Invalid parameter key >%s<!", key);
           // clang-format on
         }
-
-      list_destroy(kvlist);
     }
 }
 
@@ -338,37 +354,38 @@ Smooth(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int SMOOTH  = cdoOperatorAdd("smooth",   0,   0, NULL);
-  int SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, NULL);
+  const int SMOOTH  = cdoOperatorAdd("smooth",   0,   0, nullptr);
+  const int SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   if (operatorID == SMOOTH) smoothGetParameter(&xnsmooth, &spoint);
 
   if (spoint.radius < 0 || spoint.radius > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "radius", spoint.radius);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
   std::vector<bool> varIDs(nvars);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int gridtype = gridInqType(gridID);
-      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR)
+      const int gridID = vlistInqVarGrid(vlistID1, varID);
+      const int gridtype = gridInqType(gridID);
+      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR
+          || (operatorID == SMOOTH9 && gridtype == GRID_GENERIC && gridInqXsize(gridID) > 0 && gridInqYsize(gridID) > 0))
         {
           varIDs[varID] = true;
         }
-      else if (gridtype == GRID_UNSTRUCTURED && operatorID == SMOOTH)
+      else if (operatorID == SMOOTH && gridtype == GRID_UNSTRUCTURED)
         {
           varIDs[varID] = true;
         }
@@ -383,7 +400,7 @@ Smooth(void *process)
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (gridsizemax < spoint.maxpoints) spoint.maxpoints = gridsizemax;
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     cdoPrint("nsmooth = %d, maxpoints = %zu, radius = %gdeg, form = %s, weight0 = %g, weightR = %g", xnsmooth, spoint.maxpoints,
              spoint.radius, Form[spoint.form], spoint.weight0, spoint.weightR);
 
@@ -392,7 +409,7 @@ Smooth(void *process)
   std::vector<double> array1(gridsizemax);
   std::vector<double> array2(gridsizemax);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
   pstreamDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -437,10 +454,10 @@ Smooth(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
