@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "param_conversion.h"
 #include "pstream_int.h"
 #include "datetime.h"
 
@@ -49,16 +50,16 @@ Timselstat(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("timselrange", func_range, 0, NULL);
-  cdoOperatorAdd("timselmin",   func_min,   0, NULL);
-  cdoOperatorAdd("timselmax",   func_max,   0, NULL);
-  cdoOperatorAdd("timselsum",   func_sum,   0, NULL);
-  cdoOperatorAdd("timselmean",  func_mean,  0, NULL);
-  cdoOperatorAdd("timselavg",   func_avg,   0, NULL);
-  cdoOperatorAdd("timselvar",   func_var,   0, NULL);
-  cdoOperatorAdd("timselvar1",  func_var1,  0, NULL);
-  cdoOperatorAdd("timselstd",   func_std,   0, NULL);
-  cdoOperatorAdd("timselstd1",  func_std1,  0, NULL);
+  cdoOperatorAdd("timselrange", func_range, 0, nullptr);
+  cdoOperatorAdd("timselmin",   func_min,   0, nullptr);
+  cdoOperatorAdd("timselmax",   func_max,   0, nullptr);
+  cdoOperatorAdd("timselsum",   func_sum,   0, nullptr);
+  cdoOperatorAdd("timselmean",  func_mean,  0, nullptr);
+  cdoOperatorAdd("timselavg",   func_avg,   0, nullptr);
+  cdoOperatorAdd("timselvar",   func_var,   0, nullptr);
+  cdoOperatorAdd("timselvar1",  func_var1,  0, nullptr);
+  cdoOperatorAdd("timselstd",   func_std,   0, nullptr);
+  cdoOperatorAdd("timselstd1",  func_std1,  0, nullptr);
 
   int operatorID = cdoOperatorID();
   int operfunc = cdoOperatorF1(operatorID);
@@ -76,8 +77,9 @@ Timselstat(void *process)
   int noffset = (nargc > 1) ? parameter2int(operatorArgv()[1]) : 0;
   int nskip   = (nargc > 2) ? parameter2int(operatorArgv()[2]) : 0;
   // clang-format on
+  bool lvars2 = lvarstd || lrange;
 
-  if (cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
+  if (Options::cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
   int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
 
@@ -89,7 +91,7 @@ Timselstat(void *process)
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
   pstreamDefVlist(streamID2, vlistID2);
 
   int maxrecs = vlistNrecs(vlistID1);
@@ -102,13 +104,13 @@ Timselstat(void *process)
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  field_init(&field);
+  fieldInit(field);
   field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
 
-  Field **samp1 = field_malloc(vlistID1, FIELD_NONE);
-  Field **vars1 = field_malloc(vlistID1, FIELD_PTR);
-  Field **vars2 = NULL;
-  if (lvarstd || lrange) vars2 = field_malloc(vlistID1, FIELD_PTR);
+  FieldVector2D samp1, vars1, vars2;
+  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
+  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_PTR);
 
   for (tsID = 0; tsID < noffset; tsID++)
     {
@@ -135,7 +137,7 @@ Timselstat(void *process)
       goto LABEL_END;
     }
 
-  while (TRUE)
+  while (true)
     {
       for (nsets = 0; nsets < ndates; nsets++)
         {
@@ -155,61 +157,60 @@ Timselstat(void *process)
                   recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              Field *psamp1 = &samp1[varID][levelID];
-              Field *pvars1 = &vars1[varID][levelID];
-              Field *pvars2 = vars2 ? &vars2[varID][levelID] : NULL;
+              Field &rsamp1 = samp1[varID][levelID];
+              Field &rvars1 = vars1[varID][levelID];
 
-              size_t gridsize = pvars1->size;
+              size_t gridsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  pstreamReadRecord(streamID1, pvars1->ptr, &nmiss);
-                  pvars1->nmiss = nmiss;
+                  pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
+                  rvars1.nmiss = nmiss;
                   if (lrange)
                     {
-                      pvars2->nmiss = nmiss;
-                      for (size_t i = 0; i < gridsize; i++) pvars2->ptr[i] = pvars1->ptr[i];
+                      vars2[varID][levelID].nmiss = nmiss;
+                      for (size_t i = 0; i < gridsize; i++) vars2[varID][levelID].ptr[i] = rvars1.ptr[i];
                     }
 
-                  if (nmiss > 0 || psamp1->ptr)
+                  if (nmiss > 0 || rsamp1.ptr)
                     {
-                      if (psamp1->ptr == NULL) psamp1->ptr = (double *) Malloc(gridsize * sizeof(double));
+                      if (rsamp1.ptr == nullptr) rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
 
-                      for (size_t i = 0; i < gridsize; i++) psamp1->ptr[i] = !DBL_IS_EQUAL(pvars1->ptr[i], pvars1->missval);
+                      for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
                     }
                 }
               else
                 {
                   pstreamReadRecord(streamID1, field.ptr, &nmiss);
                   field.nmiss = nmiss;
-                  field.grid = pvars1->grid;
-                  field.missval = pvars1->missval;
+                  field.grid = rvars1.grid;
+                  field.missval = rvars1.missval;
 
-                  if (field.nmiss > 0 || psamp1->ptr)
+                  if (field.nmiss > 0 || rsamp1.ptr)
                     {
-                      if (psamp1->ptr == NULL)
+                      if (rsamp1.ptr == nullptr)
                         {
-                          psamp1->ptr = (double *) Malloc(gridsize * sizeof(double));
-                          for (size_t i = 0; i < gridsize; i++) psamp1->ptr[i] = nsets;
+                          rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
+                          for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = nsets;
                         }
 
                       for (size_t i = 0; i < gridsize; i++)
-                        if (!DBL_IS_EQUAL(field.ptr[i], pvars1->missval)) psamp1->ptr[i]++;
+                        if (!DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) rsamp1.ptr[i]++;
                     }
 
                   if (lvarstd)
                     {
-                      farsumq(pvars2, field);
-                      farsum(pvars1, field);
+                      farsumq(vars2[varID][levelID], field);
+                      farsum(rvars1, field);
                     }
                   else if (lrange)
                     {
-                      farmin(pvars2, field);
-                      farmax(pvars1, field);
+                      farmin(vars2[varID][levelID], field);
+                      farmax(rvars1, field);
                     }
                   else
                     {
-                      farfun(pvars1, field, operfunc);
+                      farfun(rvars1, field, operfunc);
                     }
                 }
             }
@@ -221,10 +222,8 @@ Timselstat(void *process)
 
                 int varID = recinfo[recID].varID;
                 int levelID = recinfo[recID].levelID;
-                Field *pvars1 = &vars1[varID][levelID];
-                Field *pvars2 = &vars2[varID][levelID];
 
-                farmoq(pvars2, *pvars1);
+                farmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
           tsID++;
@@ -238,37 +237,39 @@ Timselstat(void *process)
 
           int varID = recinfo[recID].varID;
           int levelID = recinfo[recID].levelID;
-          Field *psamp1 = &samp1[varID][levelID];
-          Field *pvars1 = &vars1[varID][levelID];
-          Field *pvars2 = vars2 ? &vars2[varID][levelID] : NULL;
+          Field &rsamp1 = samp1[varID][levelID];
+          Field &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
-              if (psamp1->ptr)
-                fardiv(pvars1, *psamp1);
+              if (rsamp1.ptr)
+                fardiv(rvars1, rsamp1);
               else
-                farcdiv(pvars1, (double) nsets);
+                farcdiv(rvars1, (double) nsets);
             }
           else if (lvarstd)
             {
-              if (psamp1->ptr)
+              Field &rvars2 = vars2[varID][levelID];
+              if (rsamp1.ptr)
                 {
                   if (lstd)
-                    farstd(pvars1, *pvars2, *psamp1, divisor);
+                    farstd(rvars1, rvars2, rsamp1, divisor);
                   else
-                    farvar(pvars1, *pvars2, *psamp1, divisor);
+                    farvar(rvars1, rvars2, rsamp1, divisor);
                 }
               else
                 {
+                  Field &rvars2 = vars2[varID][levelID];
                   if (lstd)
-                    farcstd(pvars1, *pvars2, nsets, divisor);
+                    farcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farcvar(pvars1, *pvars2, nsets, divisor);
+                    farcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
           else if (lrange)
             {
-              farsub(pvars1, *pvars2);
+              Field &rvars2 = vars2[varID][levelID];
+              farsub(rvars1, rvars2);
             }
         }
 
@@ -281,10 +282,10 @@ Timselstat(void *process)
 
           int varID = recinfo[recID].varID;
           int levelID = recinfo[recID].levelID;
-          Field *pvars1 = &vars1[varID][levelID];
+          Field &rvars1 = vars1[varID][levelID];
 
           pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, pvars1->ptr, pvars1->nmiss);
+          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
@@ -302,16 +303,16 @@ Timselstat(void *process)
 
 LABEL_END:
 
-  field_free(vars1, vlistID1);
-  field_free(samp1, vlistID1);
-  if (lvarstd) field_free(vars2, vlistID1);
+  fieldsFree(vlistID1, vars1);
+  fieldsFree(vlistID1, samp1);
+  if (lvars2) fieldsFree(vlistID1, vars2);
 
   if (field.ptr) Free(field.ptr);
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
