@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,10 +17,11 @@
 
 #include "cdo_int.h"
 #include "cdo_wtime.h"
-#include "grid.h"
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
-#include "cdoOptions.h"
+#include "cdo_options.h"
+#include "progress.h"
+#include "cimdOmp.h"
 
 #define ZERO 0.0
 #define ONE 1.0
@@ -58,10 +59,9 @@ remap_set_threshhold(double threshhold)
 */
 static void
 pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lcoinc, bool *lthresh, double beglat,
-                  double beglon, double endlat, double endlon, double *begseg, bool lrevers, long num_srch_cells,
-                  long srch_corners, const size_t *restrict srch_add, const double *restrict srch_corner_lat,
-                  const double *restrict srch_corner_lon, bool *luse_last, double *intrsct_x, double *intrsct_y,
-                  int *avoid_pole_count, double *avoid_pole_offset)
+                  double beglon, double endlat, double endlon, double *begseg, bool lrevers, long num_srch_cells, long srch_corners,
+                  const size_t *restrict srch_add, const double *restrict srch_corner_lat, const double *restrict srch_corner_lon,
+                  bool *luse_last, double *intrsct_x, double *intrsct_y, int *avoid_pole_count, double *avoid_pole_offset)
 {
   /*
     Intent(in):
@@ -111,7 +111,7 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
   *intrsct_lat = endlat;
   *intrsct_lon = endlon;
 
-  loutside = FALSE;
+  loutside = false;
   s1 = ZERO;
 
   /* Convert coordinates */
@@ -164,7 +164,7 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
      Search for location of this segment in ocean grid using cross
      product method to determine whether a point is enclosed by a cell
   */
-  while (TRUE) /* srch_loop */
+  while (true) /* srch_loop */
     {
       /* If last segment crossed threshold, use that location */
 
@@ -277,7 +277,7 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
         take some baby steps along the segment to see if any
         part of the segment lies inside the grid.
       */
-      loutside = TRUE;
+      loutside = true;
       s1 = s1 + BABY_STEP;
       x1 = begx + s1 * (x2 - begx);
       y1 = begy + s1 * (y2 - begy);
@@ -335,7 +335,7 @@ after_srch_loop:
            Return the point of intersection (adding a small
            number so the intersection is off the grid line).
       */
-      if (fabs(determ) > 1.e-30)
+      if (std::fabs(determ) > 1.e-30)
         {
           s1 = (rhs1 * mat4 - mat2 * rhs2) / determ;
           s2 = (mat1 * rhs2 - rhs1 * mat3) / determ;
@@ -391,9 +391,9 @@ after_srch_loop:
                   *intrsct_lon = rns * atan2(*intrsct_y, *intrsct_x);
                   if (*intrsct_lon < ZERO) *intrsct_lon = *intrsct_lon + PI2;
 
-                  if (fabs(*intrsct_x) > 1.e-10)
+                  if (std::fabs(*intrsct_x) > 1.e-10)
                     *intrsct_lat = (pi4 - asin(rns * HALF * (*intrsct_x) / cos(*intrsct_lon))) * TWO;
-                  else if (fabs(*intrsct_y) > 1.e-10)
+                  else if (std::fabs(*intrsct_y) > 1.e-10)
                     *intrsct_lat = (pi4 - asin(HALF * (*intrsct_y) / sin(*intrsct_lon))) * TWO;
                   else
                     *intrsct_lat = TWO * pi4;
@@ -441,7 +441,7 @@ after_srch_loop:
      (it is ok for endpoint to be pole point)
   */
 
-  if (fabs(*intrsct_x) < 1.e-10 && fabs(*intrsct_y) < 1.e-10 && (IS_NOT_EQUAL(endx, 0) && IS_NOT_EQUAL(endy, 0)))
+  if (std::fabs(*intrsct_x) < 1.e-10 && std::fabs(*intrsct_y) < 1.e-10 && (IS_NOT_EQUAL(endx, 0) && IS_NOT_EQUAL(endy, 0)))
     {
       if (*avoid_pole_count > 2)
         {
@@ -521,11 +521,11 @@ after_srch_loop:
    have already been restricted in the calling routine.
 */
 static void
-intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lcoinc, double beglat, double beglon,
-             double endlat, double endlon, double *begseg, bool lbegin, bool lrevers, long num_srch_cells, long srch_corners,
+intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lcoinc, double beglat, double beglon, double endlat,
+             double endlon, double *begseg, bool lbegin, bool lrevers, long num_srch_cells, long srch_corners,
              const size_t *restrict srch_add, const double *restrict srch_corner_lat, const double *restrict srch_corner_lon,
-             long *last_loc, bool *lthresh, double *intrsct_lat_off, double *intrsct_lon_off, bool *luse_last,
-             double *intrsct_x, double *intrsct_y, int *avoid_pole_count, double *avoid_pole_offset)
+             long *last_loc, bool *lthresh, double *intrsct_lat_off, double *intrsct_lon_off, bool *luse_last, double *intrsct_x,
+             double *intrsct_y, int *avoid_pole_count, double *avoid_pole_offset)
 {
   /*
     Intent(in):
@@ -573,8 +573,8 @@ intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lco
     {
       if (*lthresh) *location = *last_loc;
       pole_intersection(location, intrsct_lat, intrsct_lon, lcoinc, lthresh, beglat, beglon, endlat, endlon, begseg, lrevers,
-                        num_srch_cells, srch_corners, srch_add, srch_corner_lat, srch_corner_lon, luse_last, intrsct_x,
-                        intrsct_y, avoid_pole_count, avoid_pole_offset);
+                        num_srch_cells, srch_corners, srch_add, srch_corner_lat, srch_corner_lon, luse_last, intrsct_x, intrsct_y,
+                        avoid_pole_count, avoid_pole_offset);
 
       if (*lthresh)
         {
@@ -585,7 +585,7 @@ intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lco
       return;
     }
 
-  loutside = FALSE;
+  loutside = false;
   if (lbegin)
     {
       lat1 = beglat;
@@ -610,7 +610,7 @@ intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lco
      Search for location of this segment in ocean grid using cross
      product method to determine whether a point is enclosed by a cell
   */
-  while (TRUE) /* srch_loop */
+  while (true) /* srch_loop */
     {
       /* If last segment crossed threshold, use that location */
 
@@ -743,7 +743,7 @@ intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool *lco
         Take some baby steps along the segment to see if any
         part of the segment lies inside the grid.
       */
-      loutside = TRUE;
+      loutside = true;
       s1 = s1 + BABY_STEP;
       lat1 = beglat + s1 * (endlat - beglat);
       lon1 = beglon + s1 * (lon2 - beglon);
@@ -811,7 +811,7 @@ after_srch_loop:
            Return the point of intersection (adding a small
            number so the intersection is off the grid line).
       */
-      if (fabs(determ) > 1.e-30)
+      if (std::fabs(determ) > 1.e-30)
         {
           s1 = (rhs1 * mat4 - mat2 * rhs2) / determ;
           s2 = (mat1 * rhs2 - rhs1 * mat3) / determ;
@@ -952,7 +952,7 @@ phi_gradient(double in_phi1, double in_phi2, double dphi, double f1, double f2,
       else
         fac = -PI;
 
-      fint = f1 + (f2 - f1) * (fac - phi1) / fabs(dphi);
+      fint = f1 + (f2 - f1) * (fac - phi1) / std::fabs(dphi);
       weight = HALF * phi1 * (phi1 - fac) * f1 - HALF * phi2 * (phi2 + fac) * f2 + HALF * fac * (phi1 + phi2) * fint;
     }
 
@@ -1221,22 +1221,22 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   double findex = 0;
 
-  if (cdoVerbose) cdoPrint("Called %s()", __func__);
+  if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  progressInit();
+  Progress::init();
 
   long num_wts = rv.num_wts;
 
   grid_store_t *grid_store = (grid_store_t *) Malloc(sizeof(grid_store_t));
   grid_store_init(grid_store, tgt_grid->size);
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       cdoPrint("North threshhold: %g", north_thresh);
       cdoPrint("South threshhold: %g", south_thresh);
     }
 
-  double start = cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   long src_grid_size = src_grid->size;
   long tgt_grid_size = tgt_grid->size;
@@ -1271,8 +1271,8 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   for (i = 0; i < Threading::ompNumThreads; ++i)
     {
-      srch_corner_lat[i] = NULL;
-      srch_corner_lon[i] = NULL;
+      srch_corner_lat[i] = nullptr;
+      srch_corner_lon[i] = nullptr;
       max_srch_cells[i] = 0;
     }
 
@@ -1282,10 +1282,10 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   srch_corners = tgt_num_cell_corners;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none)                                                                         \
-  shared(findex, rsearch, num_wts, src_centroid_lon, src_centroid_lat, grid_store, rv, cdoVerbose, max_subseg, \
-         srch_corner_lat, srch_corner_lon, max_srch_cells, src_num_cell_corners, srch_corners,         \
-         src_grid, tgt_grid, tgt_grid_size, src_grid_size, srch_add)
+#pragma omp parallel for default(none) shared(findex, rsearch, num_wts, src_centroid_lon, src_centroid_lat, grid_store, rv, \
+                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, max_srch_cells,     \
+                                              src_num_cell_corners, srch_corners, src_grid, tgt_grid, tgt_grid_size,        \
+                                              src_grid_size, srch_add)
 #endif
   for (long src_cell_add = 0; src_cell_add < src_grid_size; ++src_cell_add)
     {
@@ -1295,7 +1295,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progressStatus(0, 0.5, findex / src_grid_size);
+      if (ompthID == 0) Progress::update(0, 0.5, findex / src_grid_size);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1317,10 +1317,8 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
       // Create search arrays
       if (num_srch_cells > max_srch_cells[ompthID])
         {
-          srch_corner_lat[ompthID]
-              = (double *) realloc(srch_corner_lat[ompthID], srch_corners * num_srch_cells * sizeof(double));
-          srch_corner_lon[ompthID]
-              = (double *) realloc(srch_corner_lon[ompthID], srch_corners * num_srch_cells * sizeof(double));
+          srch_corner_lat[ompthID] = (double *) realloc(srch_corner_lat[ompthID], srch_corners * num_srch_cells * sizeof(double));
+          srch_corner_lon[ompthID] = (double *) realloc(srch_corner_lon[ompthID], srch_corners * num_srch_cells * sizeof(double));
           max_srch_cells[ompthID] = num_srch_cells;
         }
 
@@ -1386,19 +1384,17 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
               /*  Prevent infinite loops if integration gets stuck near cell or threshold boundary */
               num_subseg++;
               if (num_subseg >= max_subseg)
-                cdoAbort("Integration stalled: num_subseg exceeded limit "
-                         "(grid1[%zu]: lon1=%g lon2=%g lat1=%g lat2=%g)!",
+                cdoAbort("Integration stalled: num_subseg exceeded limit (grid1[%zu]: lon1=%g lon2=%g lat1=%g lat2=%g)!",
                          src_cell_add, beglon, endlon, beglat, endlat);
 
               /* Uwe Schulzweida: skip very small regions */
               if (num_subseg % 1000 == 0)
                 {
-                  if (fabs(beglat - endlat) < 1.e-10 || fabs(beglon - endlon) < 1.e-10)
+                  if (std::fabs(beglat - endlat) < 1.e-10 || std::fabs(beglon - endlon) < 1.e-10)
                     {
-                      if (cdoVerbose)
-                        cdoPrint("Skip very small region (grid1[%ld]): lon=%g "
-                                 "dlon=%g lat=%g dlat=%g",
-                                 src_cell_add, beglon, endlon - beglon, beglat, endlat - beglat);
+                      if (Options::cdoVerbose)
+                        cdoPrint("Skip very small region (grid1[%ld]): lon=%g dlon=%g lat=%g dlat=%g", src_cell_add, beglon,
+                                 endlon - beglon, beglat, endlat - beglat);
                       break;
                     }
                 }
@@ -1470,8 +1466,8 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   for (i = 0; i < Threading::ompNumThreads; ++i)
     {
-      srch_corner_lat[i] = NULL;
-      srch_corner_lon[i] = NULL;
+      srch_corner_lat[i] = nullptr;
+      srch_corner_lon[i] = nullptr;
     }
 
   for (i = 0; i < Threading::ompNumThreads; ++i) max_srch_cells[i] = 0;
@@ -1482,10 +1478,10 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none)                                                                         \
-  shared(findex, rsearch, num_wts, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv, cdoVerbose, max_subseg, \
-         srch_corner_lat, srch_corner_lon, max_srch_cells, tgt_num_cell_corners, srch_corners,         \
-         src_grid, tgt_grid, tgt_grid_size, src_grid_size, srch_add)
+#pragma omp parallel for default(none) shared(findex, rsearch, num_wts, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv, \
+                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, max_srch_cells,     \
+                                              tgt_num_cell_corners, srch_corners, src_grid, tgt_grid, tgt_grid_size,        \
+                                              src_grid_size, srch_add)
 #endif
   for (long tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
@@ -1495,7 +1491,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progressStatus(0.5, 0.5, findex / tgt_grid_size);
+      if (ompthID == 0) Progress::update(0.5, 0.5, findex / tgt_grid_size);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1517,10 +1513,8 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
       // Create search arrays
       if (num_srch_cells > max_srch_cells[ompthID])
         {
-          srch_corner_lat[ompthID]
-              = (double *) realloc(srch_corner_lat[ompthID], srch_corners * num_srch_cells * sizeof(double));
-          srch_corner_lon[ompthID]
-              = (double *) realloc(srch_corner_lon[ompthID], srch_corners * num_srch_cells * sizeof(double));
+          srch_corner_lat[ompthID] = (double *) realloc(srch_corner_lat[ompthID], srch_corners * num_srch_cells * sizeof(double));
+          srch_corner_lon[ompthID] = (double *) realloc(srch_corner_lon[ompthID], srch_corners * num_srch_cells * sizeof(double));
           max_srch_cells[ompthID] = num_srch_cells;
         }
 
@@ -1584,19 +1578,17 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
               /*  Prevent infinite loops if integration gets stuck near cell or threshold boundary */
               num_subseg++;
               if (num_subseg >= max_subseg)
-                cdoAbort("Integration stalled: num_subseg exceeded limit "
-                         "(grid2[%zu]: lon1=%g lon2=%g lat1=%g lat2=%g)!",
+                cdoAbort("Integration stalled: num_subseg exceeded limit (grid2[%zu]: lon1=%g lon2=%g lat1=%g lat2=%g)!",
                          tgt_cell_add, beglon, endlon, beglat, endlat);
 
               /* Uwe Schulzweida: skip very small regions */
               if (num_subseg % 1000 == 0)
                 {
-                  if (fabs(beglat - endlat) < 1.e-10 || fabs(beglon - endlon) < 1.e-10)
+                  if (std::fabs(beglat - endlat) < 1.e-10 || std::fabs(beglon - endlon) < 1.e-10)
                     {
-                      if (cdoVerbose)
-                        cdoPrint("Skip very small region (grid2[%ld]): lon=%g "
-                                 "dlon=%g lat=%g dlat=%g",
-                                 tgt_cell_add, beglon, endlon - beglon, beglat, endlat - beglat);
+                      if (Options::cdoVerbose)
+                        cdoPrint("Skip very small region (grid2[%ld]): lon=%g dlon=%g lat=%g dlat=%g", tgt_cell_add, beglon,
+                                 endlon - beglon, beglat, endlat - beglat);
                       break;
                     }
                 }
@@ -1657,7 +1649,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
         }
     }
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
   /* Finished with all cells: deallocate search arrays */
 
@@ -1705,7 +1697,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
     {
       num_links = rv.num_links;
 
-      if ( cdoVerbose )
+      if ( Options::cdoVerbose )
         for ( long n = 0; n < num_links; n++ )
           printf("wts1: %d %g\n", n, rv.wts[3*n]);
 
@@ -1732,7 +1724,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
             }
         }
 
-     if ( cdoVerbose ) cdoPrint("Removed number of links = %zu", rv.num_links -
+     if ( Options::cdoVerbose ) cdoPrint("Removed number of links = %zu", rv.num_links -
   num_links);
 
       rv.num_links = num_links;
@@ -1744,7 +1736,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   long num_links = rv.num_links;
 
-  if (cdoVerbose) cdoPrint("Total number of links = %zu", rv.num_links);
+  if (Options::cdoVerbose) cdoPrint("Total number of links = %zu", rv.num_links);
 
   for (long n = 0; n < src_grid_size; ++n)
     if (IS_NOT_EQUAL(src_grid->cell_area[n], 0)) src_grid->cell_frac[n] /= src_grid->cell_area[n];
@@ -1796,7 +1788,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
           else if (rv.normOpt == NormOpt::NONE)
             norm_factor = tgt_grid->cell_area[n];
 
-          if (tgt_centroid_lat[n] > 0 && fabs(tgt_centroid_lat[n] - norm_factor) > .01)
+          if (tgt_centroid_lat[n] > 0 && std::fabs(tgt_centroid_lat[n] - norm_factor) > .01)
             cdoPrint("Error: sum of wts for map1 %ld %g %g", n, tgt_centroid_lat[n], norm_factor);
         }
     }  // lcheck
@@ -1806,5 +1798,5 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   Free(tgt_centroid_lat);
   Free(tgt_centroid_lon);
 
-  if (cdoVerbose) cdoPrint("Cells search: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Cells search: %.2f seconds", cdo_get_wtime() - start);
 }  // remapConservWeightsScrip
